# talKZ - (Kalfa and Zack)'s talk implementation
# Copyright (C) 2002:
# 	Cosimo Alfarano      <kalfa@cs.unibo.it>
# 	Stefano Zacchiroli   <zack@cs.unibo.it>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import os
import re
import utmp


_PATH_DEV = "/dev/"


class NotLoggedIn(Exception):
    def __init__(self, user):
        self.__user = user
    def __str__(self):
        return `self.__user`
    def getUser(self):
        return self.__user


def find_ttys(user):
    """ return a list of ttys where 'user' is currently logged """

    ttys = []
    u = utmp.UtmpRecord()
    u.setutent()
    entry = u.getutent_dict()
    while entry != {}:
        if entry["ut_user"] == user:
            ttys.append(_PATH_DEV + entry["ut_line"])
        entry = u.getutent_dict()
    u.endutent()
    return ttys


def tty_announce(tty, msg):
    """
    try to write 'msg' to tty 'tty', return 1 in case of success, 0 otherwise
    (e.g. tty isn't writable). Note that the return value is also equal to the
    number of tty on which the announce was delivered.

    'tty' argument must be a string representing a full path to a tty file
    (e.g. "/dev/tty1") _or_ a tty name as reported in utmp (e.g. "tty1")
    """

    if type(tty) != str:
        raise TypeError("'tty' must be a string")
    if not re.match(re.compile(re.escape(_PATH_DEV)), tty):
        tty = _PATH_DEV + tty
    if not os.access(tty, os.W_OK): # tty not writable
        return 0
    else:   # tty writable
        t = open(tty, "w")
        t.write(msg)
        t.close()
        return 1


def announce(user, msg, tty=None):
    """ try to write 'msg' to all ttys where 'user' is logged. Return the
    number of ttys on which the message was succesfully written. """

    if tty == None:
        ttys = find_ttys(user)
    elif type(tty) == str:
        if tty == "":
            ttys = find_ttys(user)
        else:
            ttys = [tty]
    elif type(tty) == list:
        ttys = tty
        for t in ttys:
            if type(t) != str:
                raise TypeError("tty must be a string or a list of strings")
    else:
        raise TypeError("tty must be a string or a list of strings")
    if len(ttys) == 0:  # user not logged
        raise NotLoggedIn(user)
    # return number of ttys announced
    return  \
        reduce((lambda x,y: x+y), map((lambda t: tty_announce(t, msg)), ttys))

