# talKZ - (Kalfa and Zack)'s talk implementation
# Copyright (C) 2002:
# 	Cosimo Alfarano      <kalfa@cs.unibo.it>
# 	Stefano Zacchiroli   <zack@cs.unibo.it>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

class UserInterface(object):
    """ abstraction over user interfaces for talkz clients """

    def __init__(self, logger=None):
        """ initialize user interface """
        raise NotImplementedError("UserInterface")

    def setStatusMsg(self, msg):
        """ change status message """
        raise NotImplementedError("setStatusMsg")

    def putCharL(self, char):
        """ write a char to the local window """
        raise NotImplementedError("putCharL")

    def putCharR(self, char):
        """ write a char to the remote window """
        raise NotImplementedError("putCharR")

    def putStringL(self, string):
        """ write a string to the local window """
        raise NotImplementedError("putStringL")

    def putStringR(self, string):
        """ write a string to the remote window """
        raise NotImplementedError("putStringR")

    def getChar(self):
        """ get a char from the user and return it """
        raise NotImplementedError("getChar")

    def setMagicCharsL(self, magicchars):
        """ set magic characters for local window; 'magicchars' should be a
        triple (cerase, werase, lerase) containing characters (integers)
        representing "Erase Char", "Erase Word" and "Erase Line" semantics """
        raise NotImplementedError("setMagicCharsL")

    def setMagicCharsR(self, magicchars):
        """ set magic characters for remote window; 'magicchars' should be a
        triple (cerase, werase, lerase) containing characters (integers)
        representing "Erase Char", "Erase Word" and "Erase Line" semantics """
        raise NotImplementedError("setMagicCharsR")

    def getMagicCharsL(self):
        """ return local magic chars in this order: <cerase, werase, lerase>
        """
        raise NotImplementedError("getMagicCharsL")

    def getMagicCharsR(self):
        """ return remote magic chars in this order: <cerase, werase, lerase>
        """
        raise NotImplementedError("getMagicCharsR")

    def start(self):
        """ draw user interface """
        raise NotImplementedError("start")

    def stop(self):
        """ shutdown the ui, performing some cleanup action (mainly needed for
        ncurses based ui """
        raise NotImplementedError("stop")

    def isRunning(self):
        """ return whether the UI is running or not """
        raise NotImplementedError("isRunning")

    def refresh(self):
        """ refresh user interface, only needed when, for strange reasons, user
        interface got messed up. In 'normal' behaviour other methods are able
        to refresh the needed part of the ui """
        raise NotImplementedError("refresh")

